#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "usbdrv.h"
#include "sdpgp.h"
//uint16_t cutOff=18;
//uint8_t csPort=1<<0;
uint16_t baselinezero;
uint16_t baselineone;
uint16_t baselinetwo,baselinethree;
uint16_t baselinefour,baselinefive;
uint16_t baselinesix,baselineseven;
uint16_t readPinC(uint8_t Pin)
{
    uint8_t bitmask=1<<Pin;
    PORTC &= ~(bitmask);
    DDRC |= bitmask;
    _delay_ms(1);
    DDRC &= ~(bitmask);
    uint16_t cycles = 16000;
    uint16_t i;
    for(i = 0; i < cycles; i++)
    {
        if (PINC & bitmask)
        {
                cycles = i;
                break;
        }
    }
    PORTC &= ~(bitmask);
    DDRC  |= bitmask;
    return cycles;
}
uint16_t readPinB(uint8_t Pin)
{
    uint8_t bitmask=1<<Pin;
    PORTB &= ~(bitmask);
    DDRB |= bitmask;
    _delay_ms(1);
    DDRB &= ~(bitmask);
    uint16_t cycles = 16000;
    uint16_t i;
    for(i = 0; i < cycles; i++)
    {
        if (PINB & bitmask)
        {
                cycles = i;
                break;
        }
    }
    PORTB &= ~(bitmask);
    DDRB  |= bitmask;
    return cycles;
}

uint16_t calibrateCapacitivePinC(uint8_t pinNumber){
  readPinC(pinNumber);
  _delay_ms(1);
  uint16_t averagingTotal = 0;
  uint8_t i;
  uint16_t reading;
  for (i = 0; i < 8; i++){
    reading = readPinC(pinNumber);
    averagingTotal += reading;
    _delay_ms(1);
  }
  return averagingTotal / 8;
  _delay_ms(10);
}
uint16_t calibrateCapacitivePinB(uint8_t pinNumber){
  readPinB(pinNumber);
  _delay_ms(1);
  uint16_t averagingTotal = 0;
  uint8_t i;
  uint16_t reading;
  for (i = 0; i < 8; i++){
    reading = readPinB(pinNumber);
    averagingTotal += reading;
    _delay_ms(1);
  }
  return averagingTotal / 8;
  _delay_ms(10);
}
void setupBaselines(void){
    uint16_t baseReading = calibrateCapacitivePinC(0);
    baselinezero = baseReading + (baseReading / 4);
    baseReading = calibrateCapacitivePinC(1);
    baselineone = baseReading + (baseReading / 4);
    baseReading = calibrateCapacitivePinC(4);
    baselinetwo = baseReading + (baseReading / 4);
    baseReading = calibrateCapacitivePinC(5);
    baselinethree = baseReading + (baseReading / 4);
    
    baseReading = calibrateCapacitivePinB(0);
    baselinefour = baseReading + (baseReading / 4);
    baseReading = calibrateCapacitivePinB(1);
    baselinefive = baseReading + (baseReading / 4);
    baseReading = calibrateCapacitivePinB(4);
    baselinesix = baseReading + (baseReading / 4);
    baseReading = calibrateCapacitivePinB(5);
    baselineseven = baseReading + (baseReading / 4);
}
/*
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
   // usbRequest_t *rq = (void *)data;
    usbMsgPtr = (uchar*) &oo;
    return sizeof(oo);
}*/
static unsigned char idleRate;

usbMsgLen_t usbFunctionSetup(unsigned char data[8])
{
    usbRequest_t *rq = (void *)data;
    if ((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) /* class request type */
    {
        if (rq->bRequest == USBRQ_HID_GET_REPORT) /* wValue: ReportType (highbyte), ReportID (lowbyte) */
        {
            usbMsgPtr = &oo;
            return sizeof(oo);
        }
        else if (rq->bRequest == USBRQ_HID_GET_IDLE)
        {
            usbMsgPtr = &idleRate;
            return 1;
        }
        else if (rq->bRequest == USBRQ_HID_SET_IDLE)
        {
            idleRate = rq->wValue.bytes[1];
        }
    }
    else
    {
        // no vendor specific requests implemented
    }
    return 0;
}


main()
{
    PORTC = 0b00000000;
    PORTD = 0b00000000;
    DDRD  = 0b00001000;
    usbInit();
    usbDeviceDisconnect();
    _delay_ms(300);    
    usbDeviceConnect();
    sei();    
    setupBaselines();          
    while (1)
    {
    	usbPoll();
    	if(readPinC(0)>baselinezero)
    		oo.buttons|=1<<0;
    	else
    		oo.buttons&=~(1<<0);
    	if(readPinC(1)>baselineone)
    		oo.buttons|=1<<1;
    	else
    		oo.buttons&=~(1<<1);
    	if(readPinC(4)>baselinetwo)
    		oo.buttons|=1<<2;
    	else
    		oo.buttons&=~(1<<2);
    	if(readPinC(5)>baselinethree)
    		oo.buttons|=1<<3;
    	else
    		oo.buttons&=~(1<<3);    		
        //-----------------
        usbPoll();
    	if(readPinB(0)>baselinefour)
    		oo.buttons|=1<<4;
    	else
    		oo.buttons&=~(1<<4);
    	if(readPinB(1)>baselinefive)
    		oo.buttons|=1<<5;
    	else
    		oo.buttons&=~(1<<5);
    	if(readPinB(4)>baselinesix)
    		oo.buttons|=1<<6;
    	else
    		oo.buttons&=~(1<<6);
    	if(readPinB(5)>baselineseven)
    		oo.buttons|=1<<7;
    	else
    		oo.buttons&=~(1<<7);    
        while (!usbInterruptIsReady()) {usbPoll(); }	
        usbSetInterrupt((unsigned char *)(&oo), sizeof(oo));
    }
    return 0;
}
